@@ -1,7 +1,7 @@
 /*
  * irc-channel.c - channel and private chat management for IRC plugin
  *
- * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -1454,10 +1454,18 @@ irc_channel_display_nick_back_in_pv (struct t_irc_server *server,
                                      const char *nickname)
 {
     struct t_irc_channel *ptr_channel;
+    struct t_irc_protocol_ctxt ctxt;
 
     if (!server || (!nick && !nickname))
         return;
 
+    memset (&ctxt, 0, sizeof (ctxt));
+    ctxt.server = server;
+    ctxt.nick = (nick) ? nick->name : NULL;
+    ctxt.nick_is_me = (irc_server_strcasecmp (server, ctxt.nick, server->nick) == 0);
+    ctxt.address = (nick) ? nick->host : NULL;
+    ctxt.command = strdup ("nick_back");
+
     for (ptr_channel = server->channels; ptr_channel;
          ptr_channel = ptr_channel->next_channel)
     {
@@ -1470,13 +1478,7 @@ irc_channel_display_nick_back_in_pv (struct t_irc_server *server,
                 weechat_printf_date_tags (
                     ptr_channel->buffer,
                     0,
-                    irc_protocol_tags (
-                        server,
-                        "nick_back",
-                        NULL,
-                        NULL,
-                        (nick) ? nick->name : NULL,
-                        (nick) ? nick->host : NULL),
+                    irc_protocol_tags (&ctxt, NULL),
                     _("%s%s%s %s(%s%s%s)%s is back on server"),
                     weechat_prefix ("join"),
                     irc_nick_color_for_msg (server, 1, nick, nickname),
@@ -1490,6 +1492,8 @@ irc_channel_display_nick_back_in_pv (struct t_irc_server *server,
             ptr_channel->has_quit_server = 0;
         }
     }
+
+    free (ctxt.command);
 }
 
 /*
