@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -21,6 +21,9 @@
 #define WEECHAT_PLUGIN_IRC_CONFIG_H
 
 #define IRC_CONFIG_NAME "irc"
+#define IRC_CONFIG_PRIO_NAME (TO_STR(IRC_PLUGIN_PRIORITY) "|" IRC_CONFIG_NAME)
+
+#define IRC_CONFIG_VERSION 3
 
 enum t_irc_config_look_server_buffer
 {
@@ -99,29 +102,40 @@ extern struct t_config_option *irc_config_look_color_nicks_in_nicklist;
 extern struct t_config_option *irc_config_look_color_nicks_in_server_messages;
 extern struct t_config_option *irc_config_look_color_pv_nick_like_channel;
 extern struct t_config_option *irc_config_look_ctcp_time_format;
+extern struct t_config_option *irc_config_look_display_account_message;
 extern struct t_config_option *irc_config_look_display_away;
 extern struct t_config_option *irc_config_look_display_ctcp_blocked;
 extern struct t_config_option *irc_config_look_display_ctcp_reply;
 extern struct t_config_option *irc_config_look_display_ctcp_unknown;
+extern struct t_config_option *irc_config_look_display_extended_join;
 extern struct t_config_option *irc_config_look_display_host_join;
 extern struct t_config_option *irc_config_look_display_host_join_local;
+extern struct t_config_option *irc_config_look_display_host_notice;
 extern struct t_config_option *irc_config_look_display_host_quit;
+extern struct t_config_option *irc_config_look_display_host_wallops;
 extern struct t_config_option *irc_config_look_display_join_message;
 extern struct t_config_option *irc_config_look_display_old_topic;
 extern struct t_config_option *irc_config_look_display_pv_away_once;
 extern struct t_config_option *irc_config_look_display_pv_back;
+extern struct t_config_option *irc_config_look_display_pv_nick_change;
 extern struct t_config_option *irc_config_look_display_pv_warning_address;
 extern struct t_config_option *irc_config_look_highlight_channel;
 extern struct t_config_option *irc_config_look_highlight_pv;
 extern struct t_config_option *irc_config_look_highlight_server;
 extern struct t_config_option *irc_config_look_highlight_tags_restrict;
+extern struct t_config_option *irc_config_look_ignore_tag_messages;
 extern struct t_config_option *irc_config_look_item_channel_modes_hide_args;
 extern struct t_config_option *irc_config_look_item_display_server;
 extern struct t_config_option *irc_config_look_item_nick_modes;
 extern struct t_config_option *irc_config_look_item_nick_prefix;
 extern struct t_config_option *irc_config_look_join_auto_add_chantype;
+extern struct t_config_option *irc_config_look_list_buffer;
+extern struct t_config_option *irc_config_look_list_buffer_scroll_horizontal;
+extern struct t_config_option *irc_config_look_list_buffer_sort;
+extern struct t_config_option *irc_config_look_list_buffer_topic_strip_colors;
 extern struct t_config_option *irc_config_look_msgbuffer_fallback;
 extern struct t_config_option *irc_config_look_new_channel_position;
+extern struct t_config_option *irc_config_look_new_list_position;
 extern struct t_config_option *irc_config_look_new_pv_position;
 extern struct t_config_option *irc_config_look_nick_completion_smart;
 extern struct t_config_option *irc_config_look_nick_mode;
@@ -132,10 +146,13 @@ extern struct t_config_option *irc_config_look_notice_welcome_redirect;
 extern struct t_config_option *irc_config_look_notice_welcome_tags;
 extern struct t_config_option *irc_config_look_notify_tags_ison;
 extern struct t_config_option *irc_config_look_notify_tags_whois;
+extern struct t_config_option *irc_config_look_open_pv_buffer_echo_msg;
 extern struct t_config_option *irc_config_look_part_closes_buffer;
 extern struct t_config_option *irc_config_look_pv_buffer;
 extern struct t_config_option *irc_config_look_pv_tags;
 extern struct t_config_option *irc_config_look_raw_messages;
+extern struct t_config_option *irc_config_look_typing_status_nicks;
+extern struct t_config_option *irc_config_look_typing_status_self;
 extern struct t_config_option *irc_config_look_server_buffer;
 extern struct t_config_option *irc_config_look_smart_filter;
 extern struct t_config_option *irc_config_look_smart_filter_account;
@@ -146,6 +163,7 @@ extern struct t_config_option *irc_config_look_smart_filter_join_unmask;
 extern struct t_config_option *irc_config_look_smart_filter_mode;
 extern struct t_config_option *irc_config_look_smart_filter_nick;
 extern struct t_config_option *irc_config_look_smart_filter_quit;
+extern struct t_config_option *irc_config_look_smart_filter_setname;
 extern struct t_config_option *irc_config_look_temporary_servers;
 extern struct t_config_option *irc_config_look_topic_strip_colors;
 
@@ -157,16 +175,20 @@ extern struct t_config_option *irc_config_color_item_nick_modes;
 extern struct t_config_option *irc_config_color_item_tls_version_deprecated;
 extern struct t_config_option *irc_config_color_item_tls_version_insecure;
 extern struct t_config_option *irc_config_color_item_tls_version_ok;
+extern struct t_config_option *irc_config_color_list_buffer_line_selected;
+extern struct t_config_option *irc_config_color_list_buffer_line_selected_bg;
 extern struct t_config_option *irc_config_color_message_account;
 extern struct t_config_option *irc_config_color_message_chghost;
 extern struct t_config_option *irc_config_color_message_join;
 extern struct t_config_option *irc_config_color_message_kick;
 extern struct t_config_option *irc_config_color_message_quit;
+extern struct t_config_option *irc_config_color_message_setname;
 extern struct t_config_option *irc_config_color_mirc_remap;
 extern struct t_config_option *irc_config_color_nick_prefixes;
 extern struct t_config_option *irc_config_color_notice;
 extern struct t_config_option *irc_config_color_reason_kick;
 extern struct t_config_option *irc_config_color_reason_quit;
+extern struct t_config_option *irc_config_color_term_remap;
 extern struct t_config_option *irc_config_color_topic_current;
 extern struct t_config_option *irc_config_color_topic_new;
 extern struct t_config_option *irc_config_color_topic_old;
@@ -192,6 +214,7 @@ extern struct t_config_option *irc_config_server_default[];
 extern struct t_hashtable *irc_config_hashtable_display_join_message;
 extern struct t_hashtable *irc_config_hashtable_nick_prefixes;
 extern struct t_hashtable *irc_config_hashtable_color_mirc_remap;
+extern struct t_hashtable *irc_config_hashtable_color_term_remap;
 extern char **irc_config_nicks_hide_password;
 extern int irc_config_num_nicks_hide_password;
 
