@@ -1,7 +1,7 @@
 /*
  * plugin.c - WeeChat plugins management (load/unload dynamic C libraries)
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,7 @@
 #include "../core/weechat.h"
 #include "../core/wee-arraylist.h"
 #include "../core/wee-config.h"
+#include "../core/wee-dir.h"
 #include "../core/wee-eval.h"
 #include "../core/wee-hashtable.h"
 #include "../core/wee-hdata.h"
@@ -55,6 +56,7 @@
 #include "../gui/gui-buffer.h"
 #include "../gui/gui-chat.h"
 #include "../gui/gui-color.h"
+#include "../gui/gui-completion.h"
 #include "../gui/gui-key.h"
 #include "../gui/gui-nicklist.h"
 #include "../gui/gui-window.h"
@@ -126,7 +128,7 @@ plugin_search (const char *name)
     for (ptr_plugin = weechat_plugins; ptr_plugin;
          ptr_plugin = ptr_plugin->next_plugin)
     {
-        if (string_strcasecmp (ptr_plugin->name, name) == 0)
+        if (strcmp (ptr_plugin->name, name) == 0)
             return ptr_plugin;
     }
 
@@ -164,14 +166,17 @@ plugin_check_extension_allowed (const char *filename)
     if (!config_plugin_extensions)
         return 1;
 
+    if (!filename)
+        return 0;
+
     length = strlen (filename);
     for (i = 0; i < config_num_plugin_extensions; i++)
     {
         length_ext = strlen (config_plugin_extensions[i]);
         if (length >= length_ext)
         {
-            if (string_strcasecmp (filename + length - length_ext,
-                                   config_plugin_extensions[i]) == 0)
+            if (strcmp (filename + length - length_ext,
+                        config_plugin_extensions[i]) == 0)
             {
                 /* extension allowed */
                 return 1;
@@ -196,7 +201,7 @@ plugin_check_extension_allowed (const char *filename)
 int
 plugin_check_autoload (const char *filename)
 {
-    int i, plugin_authorized, plugin_blacklisted, length, length_ext;
+    int i, length, length_ext, match;
     char *full_name, *ptr_base_name, *base_name, *plugin_name;
 
     /* by default we can auto load all plugins */
@@ -233,8 +238,8 @@ plugin_check_autoload (const char *filename)
             length_ext = strlen (config_plugin_extensions[i]);
             if (length >= length_ext)
             {
-                if (string_strcasecmp (base_name + length - length_ext,
-                                       config_plugin_extensions[i]) == 0)
+                if (strcmp (base_name + length - length_ext,
+                            config_plugin_extensions[i]) == 0)
                 {
                     plugin_name = string_strndup (base_name, length - length_ext);
                     break;
@@ -252,33 +257,13 @@ plugin_check_autoload (const char *filename)
     if (!plugin_name)
         return 1;
 
-    /* browse array and check if plugin is "authorized" or "blacklisted" */
-    plugin_authorized = 0;
-    plugin_blacklisted = 0;
-    for (i = 0; i < plugin_autoload_count; i++)
-    {
-        if (plugin_autoload_array[i][0] == '!')
-        {
-            /*
-             * negative value: it is used to "blacklist" a plugin
-             * for example with "*,!perl", all plugins are loaded, but not perl
-             */
-            if (string_match (plugin_name, plugin_autoload_array[i] + 1, 0))
-                plugin_blacklisted = 1;
-        }
-        else
-        {
-            if (string_match (plugin_name, plugin_autoload_array[i], 0))
-                plugin_authorized = 1;
-        }
-    }
+    match = string_match_list (plugin_name,
+                               (const char **)plugin_autoload_array,
+                               1);
 
     free (plugin_name);
 
-    if (plugin_blacklisted)
-        return 0;
-
-    return plugin_authorized;
+    return match;
 }
 
 /*
@@ -289,9 +274,9 @@ plugin_check_autoload (const char *filename)
  */
 
 void
-plugin_get_args (struct t_weechat_plugin *plugin,
-                 int argc, char **argv,
-                 int *plugin_argc, char ***plugin_argv)
+plugin_get_args (int argc, char **argv,
+                 int *plugin_argc, char ***plugin_argv,
+                 int *no_connect, int *no_script)
 {
     int i, temp_argc;
     char **temp_argv;
@@ -299,6 +284,9 @@ plugin_get_args (struct t_weechat_plugin *plugin,
     temp_argc = 0;
     temp_argv = NULL;
 
+    *no_connect = 0;
+    *no_script = 0;
+
     if (argc > 0)
     {
         temp_argv = malloc ((argc + 1) * sizeof (*temp_argv));
@@ -307,12 +295,16 @@ plugin_get_args (struct t_weechat_plugin *plugin,
             for (i = 0; i < argc; i++)
             {
                 if ((strcmp (argv[i], "-a") == 0)
-                    || (strcmp (argv[i], "--no-connect") == 0)
-                    || (strcmp (argv[i], "-s") == 0)
-                    || (strcmp (argv[i], "--no-script") == 0)
-                    || (strcmp (argv[i], "--upgrade") == 0)
-                    || (strncmp (argv[i], plugin->name,
-                                 strlen (plugin->name)) == 0))
+                    || (strcmp (argv[i], "--no-connect") == 0))
+                {
+                    *no_connect = 1;
+                }
+                else if ((strcmp (argv[i], "-s") == 0)
+                         || (strcmp (argv[i], "--no-script") == 0))
+                {
+                    *no_script = 1;
+                }
+                else if (argv[i][0] != '-')
                 {
                     temp_argv[temp_argc++] = argv[i];
                 }
@@ -343,7 +335,8 @@ int
 plugin_call_init (struct t_weechat_plugin *plugin, int argc, char **argv)
 {
     t_weechat_init_func *init_func;
-    int plugin_argc, rc;
+    int no_connect, rc, old_auto_connect, no_script, old_auto_load_scripts;
+    int plugin_argc;
     char **plugin_argv;
 
     if (plugin->initialized)
@@ -355,7 +348,14 @@ plugin_call_init (struct t_weechat_plugin *plugin, int argc, char **argv)
         return 0;
 
     /* get arguments for the plugin */
-    plugin_get_args (plugin, argc, argv, &plugin_argc, &plugin_argv);
+    plugin_get_args (argc, argv,
+                     &plugin_argc, &plugin_argv, &no_connect, &no_script);
+
+    old_auto_connect = weechat_auto_connect;
+    weechat_auto_connect = (no_connect) ? 0 : 1;
+
+    old_auto_load_scripts = weechat_auto_load_scripts;
+    weechat_auto_load_scripts = (no_script) ? 0 : 1;
 
     /* init plugin */
     if (weechat_debug_core >= 1)
@@ -374,12 +374,14 @@ plugin_call_init (struct t_weechat_plugin *plugin, int argc, char **argv)
     else
     {
         gui_chat_printf (NULL,
-                         _("%sError: unable to initialize plugin "
-                           "\"%s\""),
+                         _("%sUnable to initialize plugin \"%s\""),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          plugin->filename);
     }
 
+    weechat_auto_connect = old_auto_connect;
+    weechat_auto_load_scripts = old_auto_load_scripts;
+
     if (plugin_argv)
         free (plugin_argv);
 
@@ -421,9 +423,9 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (!handle)
     {
         gui_chat_printf (NULL,
-                         _("%sError: unable to load plugin \"%s\": %s"),
+                         _("%sUnable to load plugin \"%s\": %s"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
-                         filename, dlerror());
+                         filename, dlerror ());
         gui_chat_printf (NULL,
                          _("%sIf you're trying to load a script and not a C "
                            "plugin, try command to load scripts (/perl, "
@@ -437,8 +439,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (!name)
     {
         gui_chat_printf (NULL,
-                         _("%sError: symbol \"%s\" not found in "
-                           "plugin \"%s\", failed to load"),
+                         _("%sSymbol \"%s\" not found in plugin \"%s\", "
+                           "failed to load"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          "weechat_plugin_name",
                          filename);
@@ -451,8 +453,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (!api_version)
     {
         gui_chat_printf (NULL,
-                         _("%sError: symbol \"%s\" not found in "
-                           "plugin \"%s\", failed to load"),
+                         _("%sSymbol \"%s\" not found in plugin \"%s\", "
+                           "failed to load"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          "weechat_plugin_api_version",
                          filename);
@@ -467,8 +469,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (strcmp (api_version, WEECHAT_PLUGIN_API_VERSION) != 0)
     {
         gui_chat_printf (NULL,
-                         _("%sError: API mismatch for plugin \"%s\" (current "
-                           "API: \"%s\", plugin API: \"%s\"), failed to load"),
+                         _("%sAPI mismatch for plugin \"%s\" (current API: "
+                           "\"%s\", plugin API: \"%s\"), failed to load"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          filename,
                          WEECHAT_PLUGIN_API_VERSION,
@@ -486,8 +488,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (plugin_search (name))
     {
         gui_chat_printf (NULL,
-                         _("%sError: unable to load plugin \"%s\": a plugin "
-                           "with same name already exists"),
+                         _("%sUnable to load plugin \"%s\": a plugin with "
+                           "same name already exists"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          filename);
         dlclose (handle);
@@ -499,8 +501,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (!description)
     {
         gui_chat_printf (NULL,
-                         _("%sError: symbol \"%s\" not found "
-                           "in plugin \"%s\", failed to load"),
+                         _("%sSymbol \"%s\" not found in plugin \"%s\", "
+                           "failed to load"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          "weechat_plugin_description",
                          filename);
@@ -513,8 +515,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (!author)
     {
         gui_chat_printf (NULL,
-                         _("%sError: symbol \"%s\" not found "
-                           "in plugin \"%s\", failed to load"),
+                         _("%sSymbol \"%s\" not found in plugin \"%s\", "
+                           "failed to load"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          "weechat_plugin_author",
                          filename);
@@ -527,8 +529,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (!version)
     {
         gui_chat_printf (NULL,
-                         _("%sError: symbol \"%s\" not found in "
-                           "plugin \"%s\", failed to load"),
+                         _("%sSymbol \"%s\" not found in plugin \"%s\", "
+                           "failed to load"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          "weechat_plugin_version",
                          filename);
@@ -541,8 +543,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (!license)
     {
         gui_chat_printf (NULL,
-                         _("%sError: symbol \"%s\" not found in "
-                           "plugin \"%s\", failed to load"),
+                         _("%sSymbol \"%s\" not found in plugin \"%s\", "
+                           "failed to load"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          "weechat_plugin_license",
                          filename);
@@ -558,8 +560,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     if (!init_func)
     {
         gui_chat_printf (NULL,
-                         _("%sError: function \"%s\" not "
-                           "found in plugin \"%s\", failed to load"),
+                         _("%sFunction \"%s\" not found in plugin \"%s\", "
+                           "failed to load"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          "weechat_plugin_init",
                          filename);
@@ -592,6 +594,11 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->initialized = 0;
         ptr_option = config_weechat_debug_get (name);
         new_plugin->debug = (ptr_option) ? CONFIG_INTEGER(ptr_option) : 0;
+        new_plugin->upgrading = weechat_upgrading;
+        new_plugin->variables = hashtable_new (
+            32,
+            WEECHAT_HASHTABLE_STRING, WEECHAT_HASHTABLE_STRING,
+            NULL, NULL);
 
         /* functions */
         new_plugin->plugin_get_name = &plugin_get_name;
@@ -602,8 +609,13 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->gettext = &plugin_api_gettext;
         new_plugin->ngettext = &plugin_api_ngettext;
         new_plugin->strndup = &string_strndup;
+        new_plugin->string_cut = &string_cut;
         new_plugin->string_tolower = &string_tolower;
         new_plugin->string_toupper = &string_toupper;
+        new_plugin->string_charcmp = &string_charcmp;
+        new_plugin->string_charcasecmp = &string_charcasecmp;
+        new_plugin->strcmp = &string_strcmp;
+        new_plugin->strncmp = &string_strncmp;
         new_plugin->strcasecmp = &string_strcasecmp;
         new_plugin->strcasecmp_range = &string_strcasecmp_range;
         new_plugin->strncasecmp = &string_strncasecmp;
@@ -612,6 +624,7 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->strcasestr = &string_strcasestr;
         new_plugin->strlen_screen = &gui_chat_strlen_screen;
         new_plugin->string_match = &string_match;
+        new_plugin->string_match_list = &string_match_list;
         new_plugin->string_replace = &string_replace;
         new_plugin->string_expand_home = &string_expand_home;
         new_plugin->string_eval_path_home = &string_eval_path_home;
@@ -624,19 +637,28 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->string_has_highlight = &string_has_highlight;
         new_plugin->string_has_highlight_regex = &string_has_highlight_regex;
         new_plugin->string_replace_regex = &string_replace_regex;
+        new_plugin->string_translate_chars = &string_translate_chars;
         new_plugin->string_split = &string_split;
         new_plugin->string_split_shell = &string_split_shell;
         new_plugin->string_free_split = &string_free_split;
-        new_plugin->string_build_with_split_string = &string_build_with_split_string;
+        new_plugin->string_rebuild_split_string = &string_rebuild_split_string;
         new_plugin->string_split_command = &string_split_command;
         new_plugin->string_free_split_command = &string_free_split_command;
         new_plugin->string_format_size = &string_format_size;
+        new_plugin->string_parse_size = &string_parse_size;
+        new_plugin->string_color_code_size = &gui_color_code_size;
         new_plugin->string_remove_color = &gui_color_decode;
-        new_plugin->string_encode_base64 = &string_encode_base64;
-        new_plugin->string_decode_base64 = &string_decode_base64;
+        new_plugin->string_base_encode = &string_base_encode;
+        new_plugin->string_base_decode = &string_base_decode;
+        new_plugin->string_hex_dump = &string_hex_dump;
         new_plugin->string_is_command_char = &string_is_command_char;
         new_plugin->string_input_for_buffer = &string_input_for_buffer;
         new_plugin->string_eval_expression = &eval_expression;
+        new_plugin->string_dyn_alloc = &string_dyn_alloc;
+        new_plugin->string_dyn_copy = &string_dyn_copy;
+        new_plugin->string_dyn_concat = &string_dyn_concat;
+        new_plugin->string_dyn_free = &string_dyn_free;
+        new_plugin->string_concat = &string_concat;
 
         new_plugin->utf8_has_8bits = &utf8_has_8bits;
         new_plugin->utf8_is_valid = &utf8_is_valid;
@@ -648,24 +670,32 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->utf8_strlen = &utf8_strlen;
         new_plugin->utf8_strnlen = &utf8_strnlen;
         new_plugin->utf8_strlen_screen = &utf8_strlen_screen;
-        new_plugin->utf8_charcmp = &utf8_charcmp;
-        new_plugin->utf8_charcasecmp = &utf8_charcasecmp;
         new_plugin->utf8_char_size_screen = &utf8_char_size_screen;
         new_plugin->utf8_add_offset = &utf8_add_offset;
         new_plugin->utf8_real_pos = &utf8_real_pos;
         new_plugin->utf8_pos = &utf8_pos;
         new_plugin->utf8_strndup = &utf8_strndup;
+        new_plugin->utf8_strncpy = &utf8_strncpy;
 
-        new_plugin->mkdir_home = &util_mkdir_home;
-        new_plugin->mkdir = &util_mkdir;
-        new_plugin->mkdir_parents = &util_mkdir_parents;
-        new_plugin->exec_on_files = &util_exec_on_files;
-        new_plugin->file_get_content = &util_file_get_content;
+        new_plugin->crypto_hash = &plugin_api_crypto_hash;
+        new_plugin->crypto_hash_file = &plugin_api_crypto_hash_file;
+        new_plugin->crypto_hash_pbkdf2 = &plugin_api_crypto_hash_pbkdf2;
+        new_plugin->crypto_hmac = &plugin_api_crypto_hmac;
+
+        new_plugin->mkdir_home = &dir_mkdir_home;
+        new_plugin->mkdir = &dir_mkdir;
+        new_plugin->mkdir_parents = &dir_mkdir_parents;
+        new_plugin->exec_on_files = &dir_exec_on_files;
+        new_plugin->file_get_content = &dir_file_get_content;
+        new_plugin->file_copy = &dir_file_copy;
+        new_plugin->file_compress = &dir_file_compress;
 
         new_plugin->util_timeval_cmp = &util_timeval_cmp;
         new_plugin->util_timeval_diff = &util_timeval_diff;
         new_plugin->util_timeval_add = &util_timeval_add;
         new_plugin->util_get_time_string = &util_get_time_string;
+        new_plugin->util_strftimeval = &util_strftimeval;
+        new_plugin->util_parse_time = &util_parse_time;
         new_plugin->util_version_number = &util_version_number;
 
         new_plugin->list_new = &weelist_new;
@@ -679,11 +709,22 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->list_next = &weelist_next;
         new_plugin->list_prev = &weelist_prev;
         new_plugin->list_string = &weelist_string;
+        new_plugin->list_user_data = &weelist_user_data;
         new_plugin->list_size = &weelist_size;
         new_plugin->list_remove = &weelist_remove;
         new_plugin->list_remove_all = &weelist_remove_all;
         new_plugin->list_free = &weelist_free;
 
+        new_plugin->arraylist_new = arraylist_new;
+        new_plugin->arraylist_size = arraylist_size;
+        new_plugin->arraylist_get = arraylist_get;
+        new_plugin->arraylist_search = arraylist_search;
+        new_plugin->arraylist_insert = arraylist_insert;
+        new_plugin->arraylist_add = arraylist_add;
+        new_plugin->arraylist_remove = arraylist_remove;
+        new_plugin->arraylist_clear = arraylist_clear;
+        new_plugin->arraylist_free = arraylist_free;
+
         new_plugin->hashtable_new = &hashtable_new;
         new_plugin->hashtable_set_with_size = &hashtable_set_with_size;
         new_plugin->hashtable_set = &hashtable_set;
@@ -696,11 +737,13 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->hashtable_get_string = &hashtable_get_string;
         new_plugin->hashtable_set_pointer = &hashtable_set_pointer;
         new_plugin->hashtable_add_to_infolist = &hashtable_add_to_infolist;
+        new_plugin->hashtable_add_from_infolist = &hashtable_add_from_infolist;
         new_plugin->hashtable_remove = &hashtable_remove;
         new_plugin->hashtable_remove_all = &hashtable_remove_all;
         new_plugin->hashtable_free = &hashtable_free;
 
         new_plugin->config_new = &config_file_new;
+        new_plugin->config_set_version = &config_file_set_version;
         new_plugin->config_new_section = &config_file_new_section;
         new_plugin->config_search_section = &config_file_search_section;
         new_plugin->config_new_option = &config_file_new_option;
@@ -713,6 +756,7 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->config_option_set_null = &config_file_option_set_null;
         new_plugin->config_option_unset = &config_file_option_unset;
         new_plugin->config_option_rename = &config_file_option_rename;
+        new_plugin->config_option_get_string = &config_file_option_get_string;
         new_plugin->config_option_get_pointer = &config_file_option_get_pointer;
         new_plugin->config_option_is_null = &config_file_option_is_null;
         new_plugin->config_option_default_is_null = &config_file_option_default_is_null;
@@ -720,6 +764,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->config_boolean_default = &config_file_option_boolean_default;
         new_plugin->config_integer = &config_file_option_integer;
         new_plugin->config_integer_default = &config_file_option_integer_default;
+        new_plugin->config_enum = &config_file_option_enum;
+        new_plugin->config_enum_default = &config_file_option_enum_default;
         new_plugin->config_string = &config_file_option_string;
         new_plugin->config_string_default = &config_file_option_string_default;
         new_plugin->config_color = &config_file_option_color;
@@ -729,7 +775,7 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->config_write = &config_file_write;
         new_plugin->config_read = &config_file_read;
         new_plugin->config_reload = &config_file_reload;
-        new_plugin->config_option_free = &config_file_option_free;
+        new_plugin->config_option_free = &plugin_api_config_file_option_free;
         new_plugin->config_section_free_options = &config_file_section_free_options;
         new_plugin->config_section_free = &config_file_section_free;
         new_plugin->config_free = &config_file_free;
@@ -745,8 +791,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
 
         new_plugin->prefix = &plugin_api_prefix;
         new_plugin->color = &plugin_api_color;
-        new_plugin->printf_date_tags = &gui_chat_printf_date_tags;
-        new_plugin->printf_y = &gui_chat_printf_y;
+        new_plugin->printf_datetime_tags = &gui_chat_printf_datetime_tags;
+        new_plugin->printf_y_datetime_tags = &gui_chat_printf_y_datetime_tags;
         new_plugin->log_printf = &log_printf;
 
         new_plugin->hook_command = &hook_command;
@@ -755,7 +801,9 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->hook_fd = &hook_fd;
         new_plugin->hook_process = &hook_process;
         new_plugin->hook_process_hashtable = &hook_process_hashtable;
+        new_plugin->hook_url = &hook_url;
         new_plugin->hook_connect = &hook_connect;
+        new_plugin->hook_line = &hook_line;
         new_plugin->hook_print = &hook_print;
         new_plugin->hook_signal = &hook_signal;
         new_plugin->hook_signal_send = &hook_signal_send;
@@ -763,8 +811,8 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->hook_hsignal_send = &hook_hsignal_send;
         new_plugin->hook_config = &hook_config;
         new_plugin->hook_completion = &hook_completion;
-        new_plugin->hook_completion_get_string = &hook_completion_get_string;
-        new_plugin->hook_completion_list_add = &hook_completion_list_add;
+        new_plugin->hook_completion_get_string = &gui_completion_get_string;
+        new_plugin->hook_completion_list_add = &gui_completion_list_add;
         new_plugin->hook_modifier = &hook_modifier;
         new_plugin->hook_modifier_exec = &hook_modifier_exec;
         new_plugin->hook_info = &hook_info;
@@ -777,6 +825,7 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->unhook_all = &unhook_all_plugin;
 
         new_plugin->buffer_new = &gui_buffer_new;
+        new_plugin->buffer_new_props = &gui_buffer_new_props;
         new_plugin->buffer_search = &gui_buffer_search_by_name;
         new_plugin->buffer_search_main = &gui_buffer_search_main;
         new_plugin->buffer_clear = &gui_buffer_clear;
@@ -825,6 +874,13 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->bar_remove = &gui_bar_free;
 
         new_plugin->command = &plugin_api_command;
+        new_plugin->command_options = &plugin_api_command_options;
+
+        new_plugin->completion_new = &gui_completion_new;
+        new_plugin->completion_search = &gui_completion_search;
+        new_plugin->completion_get_string = &gui_completion_get_string;
+        new_plugin->completion_list_add = &gui_completion_list_add;
+        new_plugin->completion_free = &gui_completion_free;
 
         new_plugin->network_pass_proxy = &network_pass_proxy;
         new_plugin->network_connect_to = &network_connect_to;
@@ -875,6 +931,7 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         new_plugin->hdata_pointer = &hdata_pointer;
         new_plugin->hdata_time = &hdata_time;
         new_plugin->hdata_hashtable = &hdata_hashtable;
+        new_plugin->hdata_compare = &hdata_compare;
         new_plugin->hdata_set = &hdata_set;
         new_plugin->hdata_update = &hdata_update;
         new_plugin->hdata_get_string = &hdata_get_string;
@@ -887,7 +944,7 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
         /* add new plugin to list */
         new_plugin->prev_plugin = last_weechat_plugin;
         new_plugin->next_plugin = NULL;
-        if (weechat_plugins)
+        if (last_weechat_plugin)
             last_weechat_plugin->next_plugin = new_plugin;
         else
             weechat_plugins = new_plugin;
@@ -911,8 +968,7 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
     else
     {
         gui_chat_printf (NULL,
-                         _("%sError: unable to load plugin \"%s\" "
-                           "(not enough memory)"),
+                         _("%sUnable to load plugin \"%s\" (not enough memory)"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          filename);
         dlclose (handle);
@@ -938,11 +994,11 @@ plugin_load (const char *filename, int init_plugin, int argc, char **argv)
  */
 
 void
-plugin_auto_load_file (void *args, const char *filename)
+plugin_auto_load_file (void *data, const char *filename)
 {
     struct t_plugin_args *plugin_args;
 
-    plugin_args = (struct t_plugin_args *)args;
+    plugin_args = (struct t_plugin_args *)data;
 
     if (plugin_check_extension_allowed (filename))
         plugin_load (filename, 0, plugin_args->argc, plugin_args->argv);
@@ -953,7 +1009,8 @@ plugin_auto_load_file (void *args, const char *filename)
  */
 
 int
-plugin_arraylist_cmp_cb (void *data, struct t_arraylist *arraylist,
+plugin_arraylist_cmp_cb (void *data,
+                         struct t_arraylist *arraylist,
                          void *pointer1, void *pointer2)
 {
     struct t_weechat_plugin *plugin1, *plugin2;
@@ -974,12 +1031,18 @@ plugin_arraylist_cmp_cb (void *data, struct t_arraylist *arraylist,
  */
 
 void
-plugin_auto_load (int argc, char **argv)
+plugin_auto_load (char *force_plugin_autoload,
+                  int load_from_plugin_path,
+                  int load_from_extra_lib_dir,
+                  int load_from_lib_dir,
+                  int argc, char **argv)
 {
-    char *dir_name, *plugin_path, *plugin_path2;
+    char *dir_name, *plugin_path, *extra_libdir;
+    const char *ptr_plugin_autoload;
     struct t_weechat_plugin *ptr_plugin;
     struct t_plugin_args plugin_args;
     struct t_arraylist *arraylist;
+    struct t_hashtable *options;
     int length, i;
 
     plugin_args.argc = argc;
@@ -988,42 +1051,73 @@ plugin_auto_load (int argc, char **argv)
     plugin_autoload_array = NULL;
     plugin_autoload_count = 0;
 
-    if (CONFIG_STRING(config_plugin_autoload)
-        && CONFIG_STRING(config_plugin_autoload)[0])
+    ptr_plugin_autoload = (force_plugin_autoload) ?
+        force_plugin_autoload : CONFIG_STRING(config_plugin_autoload);
+
+    if (ptr_plugin_autoload && ptr_plugin_autoload[0])
     {
-        plugin_autoload_array = string_split (CONFIG_STRING(config_plugin_autoload),
-                                              ",", 0, 0,
-                                              &plugin_autoload_count);
+        plugin_autoload_array = string_split (
+            ptr_plugin_autoload,
+            ",",
+            NULL,
+            WEECHAT_STRING_SPLIT_STRIP_LEFT
+            | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+            | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+            0,
+            &plugin_autoload_count);
     }
 
-    /* auto-load plugins in WeeChat home dir */
-    if (CONFIG_STRING(config_plugin_path)
+    /* auto-load plugins in custom path */
+    if (load_from_plugin_path
+        && CONFIG_STRING(config_plugin_path)
         && CONFIG_STRING(config_plugin_path)[0])
     {
-        plugin_path = string_expand_home (CONFIG_STRING(config_plugin_path));
-        plugin_path2 = string_replace ((plugin_path) ?
-                                       plugin_path : CONFIG_STRING(config_plugin_path),
-                                       "%h", weechat_home);
-        util_exec_on_files ((plugin_path2) ?
-                            plugin_path2 : ((plugin_path) ?
-                                            plugin_path : CONFIG_STRING(config_plugin_path)),
-                            0,
-                            &plugin_args,
-                            &plugin_auto_load_file);
+        options = hashtable_new (
+            32,
+            WEECHAT_HASHTABLE_STRING,
+            WEECHAT_HASHTABLE_STRING,
+            NULL, NULL);
+        if (options)
+            hashtable_set (options, "directory", "data");
+        plugin_path = string_eval_path_home (CONFIG_STRING(config_plugin_path),
+                                             NULL, NULL, options);
+        if (options)
+            hashtable_free (options);
         if (plugin_path)
+        {
+            dir_exec_on_files (plugin_path, 1, 0,
+                               &plugin_auto_load_file, &plugin_args);
             free (plugin_path);
-        if (plugin_path2)
-            free (plugin_path2);
+        }
+    }
+
+    /* auto-load plugins in WEECHAT_EXTRA_LIBDIR environment variable */
+    if (load_from_extra_lib_dir)
+    {
+        extra_libdir = getenv (WEECHAT_EXTRA_LIBDIR);
+        if (extra_libdir && extra_libdir[0])
+        {
+            length = strlen (extra_libdir) + 16 + 1;
+            dir_name = malloc (length);
+            snprintf (dir_name, length, "%s/plugins", extra_libdir);
+            dir_exec_on_files (dir_name, 1, 0,
+                               &plugin_auto_load_file, &plugin_args);
+            free (dir_name);
+        }
     }
 
     /* auto-load plugins in WeeChat global lib dir */
-    length = strlen (WEECHAT_LIBDIR) + 16 + 1;
-    dir_name = malloc (length);
-    if (dir_name)
+    if (load_from_lib_dir)
     {
-        snprintf (dir_name, length, "%s/plugins", WEECHAT_LIBDIR);
-        util_exec_on_files (dir_name, 0, &plugin_args, &plugin_auto_load_file);
-        free (dir_name);
+        length = strlen (WEECHAT_LIBDIR) + 16 + 1;
+        dir_name = malloc (length);
+        if (dir_name)
+        {
+            snprintf (dir_name, length, "%s/plugins", WEECHAT_LIBDIR);
+            dir_exec_on_files (dir_name, 1, 0,
+                               &plugin_auto_load_file, &plugin_args);
+            free (dir_name);
+        }
     }
 
     /* free autoload array */
@@ -1075,6 +1169,9 @@ plugin_remove (struct t_weechat_plugin *plugin)
     struct t_weechat_plugin *new_weechat_plugins;
     struct t_gui_buffer *ptr_buffer, *next_buffer;
 
+    /* remove all completions (only those created by API) */
+    gui_completion_free_all_plugin (plugin);
+
     /* close buffers created by this plugin */
     ptr_buffer = gui_buffers;
     while (ptr_buffer)
@@ -1105,7 +1202,7 @@ plugin_remove (struct t_weechat_plugin *plugin)
     config_file_free_all_plugin (plugin);
 
     /* remove all hooks */
-    unhook_all_plugin (plugin);
+    unhook_all_plugin (plugin, NULL);
 
     /* remove all infolists */
     infolist_free_all_plugin (plugin);
@@ -1133,6 +1230,7 @@ plugin_remove (struct t_weechat_plugin *plugin)
         free (plugin->license);
     if (plugin->charset)
         free (plugin->charset);
+    hashtable_free (plugin->variables);
 
     free (plugin);
 
@@ -1187,7 +1285,7 @@ plugin_unload_name (const char *name)
     else
     {
         gui_chat_printf (NULL,
-                         _("%sError: plugin \"%s\" not found"),
+                         _("%sPlugin \"%s\" not found"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          name);
     }
@@ -1241,7 +1339,7 @@ plugin_reload_name (const char *name, int argc, char **argv)
     else
     {
         gui_chat_printf (NULL,
-                         _("%sError: plugin \"%s\" not found"),
+                         _("%sPlugin \"%s\" not found"),
                          gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
                          name);
     }
@@ -1303,20 +1401,17 @@ plugin_display_short_list ()
  */
 
 void
-plugin_init (int auto_load, int argc, char *argv[])
+plugin_init (char *force_plugin_autoload, int argc, char *argv[])
 {
     /* read plugins options on disk */
     plugin_config_init ();
     plugin_config_read ();
 
-    /* auto-load plugins if asked */
-    if (auto_load)
-    {
-        plugin_quiet = 1;
-        plugin_auto_load (argc, argv);
-        plugin_display_short_list ();
-        plugin_quiet = 0;
-    }
+    /* auto-load plugins */
+    plugin_quiet = 1;
+    plugin_auto_load (force_plugin_autoload, 1, 1, 1, argc, argv);
+    plugin_display_short_list ();
+    plugin_quiet = 0;
 }
 
 /*
@@ -1341,11 +1436,13 @@ plugin_end ()
  */
 
 struct t_hdata *
-plugin_hdata_plugin_cb (void *data, const char *hdata_name)
+plugin_hdata_plugin_cb (const void *pointer, void *data,
+                        const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, "prev_plugin", "next_plugin",
@@ -1363,6 +1460,8 @@ plugin_hdata_plugin_cb (void *data, const char *hdata_name)
         HDATA_VAR(struct t_weechat_plugin, priority, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_weechat_plugin, initialized, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_weechat_plugin, debug, INTEGER, 0, NULL, NULL);
+        HDATA_VAR(struct t_weechat_plugin, upgrading, INTEGER, 0, NULL, NULL);
+        HDATA_VAR(struct t_weechat_plugin, variables, HASHTABLE, 0, NULL, NULL);
         HDATA_VAR(struct t_weechat_plugin, prev_plugin, POINTER, 0, NULL, hdata_name);
         HDATA_VAR(struct t_weechat_plugin, next_plugin, POINTER, 0, NULL, hdata_name);
         HDATA_LIST(weechat_plugins, WEECHAT_HDATA_LIST_CHECK_POINTERS);
@@ -1420,6 +1519,10 @@ plugin_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!infolist_new_var_integer (ptr_item, "debug", plugin->debug))
         return 0;
+    if (!infolist_new_var_integer (ptr_item, "upgrading", plugin->upgrading))
+        return 0;
+    if (!hashtable_add_to_infolist (plugin->variables, ptr_item, "var"))
+        return 0;
 
     return 1;
 }
@@ -1442,11 +1545,15 @@ plugin_print_log ()
         log_printf ("  handle . . . . . . . . : 0x%lx", ptr_plugin->handle);
         log_printf ("  name . . . . . . . . . : '%s'",  ptr_plugin->name);
         log_printf ("  description. . . . . . : '%s'",  ptr_plugin->description);
+        log_printf ("  author . . . . . . . . : '%s'",  ptr_plugin->author);
         log_printf ("  version. . . . . . . . : '%s'",  ptr_plugin->version);
+        log_printf ("  license. . . . . . . . : '%s'",  ptr_plugin->license);
         log_printf ("  charset. . . . . . . . : '%s'",  ptr_plugin->charset);
         log_printf ("  priority . . . . . . . : %d",    ptr_plugin->priority);
         log_printf ("  initialized. . . . . . : %d",    ptr_plugin->initialized);
         log_printf ("  debug. . . . . . . . . : %d",    ptr_plugin->debug);
+        log_printf ("  upgrading. . . . . . . : %d",    ptr_plugin->upgrading);
+        hashtable_print_log (ptr_plugin->variables, "variables");
         log_printf ("  prev_plugin. . . . . . : 0x%lx", ptr_plugin->prev_plugin);
         log_printf ("  next_plugin. . . . . . : 0x%lx", ptr_plugin->next_plugin);
     }
