@@ -1,7 +1,7 @@
 /*
  * relay-command.c - relay command
  *
- * Copyright (C) 2003-2022 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -204,25 +204,25 @@ relay_command_relay (const void *pointer, void *data,
 
     if (argc > 1)
     {
-        if (weechat_strcasecmp (argv[1], "list") == 0)
+        if (weechat_strcmp (argv[1], "list") == 0)
         {
             relay_command_client_list (0);
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "listfull") == 0)
+        if (weechat_strcmp (argv[1], "listfull") == 0)
         {
             relay_command_client_list (1);
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "listrelay") == 0)
+        if (weechat_strcmp (argv[1], "listrelay") == 0)
         {
             relay_command_server_list ();
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "add") == 0)
+        if (weechat_strcmp (argv[1], "add") == 0)
         {
             WEECHAT_COMMAND_MIN_ARGS(4, "add");
             relay_server_get_protocol_args (argv[2], NULL, NULL, NULL,
@@ -245,7 +245,7 @@ relay_command_relay (const void *pointer, void *data,
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "del") == 0)
+        if (weechat_strcmp (argv[1], "del") == 0)
         {
             WEECHAT_COMMAND_MIN_ARGS(3, "del");
             ptr_server = relay_server_search (argv_eol[2]);
@@ -279,7 +279,7 @@ relay_command_relay (const void *pointer, void *data,
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "stop") == 0)
+        if (weechat_strcmp (argv[1], "stop") == 0)
         {
             WEECHAT_COMMAND_MIN_ARGS(3, "stop");
             ptr_server = relay_server_search (argv_eol[2]);
@@ -298,7 +298,7 @@ relay_command_relay (const void *pointer, void *data,
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "start") == 0)
+        if (weechat_strcmp (argv[1], "start") == 0)
         {
             WEECHAT_COMMAND_MIN_ARGS(3, "start");
             ptr_server = relay_server_search (argv_eol[2]);
@@ -318,7 +318,7 @@ relay_command_relay (const void *pointer, void *data,
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "restart") == 0)
+        if (weechat_strcmp (argv[1], "restart") == 0)
         {
             WEECHAT_COMMAND_MIN_ARGS(3, "restart");
             ptr_server = relay_server_search (argv_eol[2]);
@@ -338,19 +338,19 @@ relay_command_relay (const void *pointer, void *data,
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "raw") == 0)
+        if (weechat_strcmp (argv[1], "raw") == 0)
         {
             relay_raw_open (1);
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "sslcertkey") == 0)
+        if (weechat_strcmp (argv[1], "tlscertkey") == 0)
         {
-            relay_network_set_ssl_cert_key (1);
+            relay_network_set_tls_cert_key (1);
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "up") == 0)
+        if (weechat_strcmp (argv[1], "up") == 0)
         {
             if (relay_buffer && (relay_buffer_selected_line > 0))
             {
@@ -360,7 +360,7 @@ relay_command_relay (const void *pointer, void *data,
             return WEECHAT_RC_OK;
         }
 
-        if (weechat_strcasecmp (argv[1], "down") == 0)
+        if (weechat_strcmp (argv[1], "down") == 0)
         {
             if (relay_buffer
                 && relay_buffer_selected_line < relay_client_count - 1)
@@ -396,68 +396,61 @@ relay_command_init ()
     weechat_hook_command (
         "relay",
         N_("relay control"),
+        /* TRANSLATORS: only text between angle brackets (eg: "<name>") must be translated */
         N_("list|listfull|listrelay"
            " || add <name> <port>|<path>"
            " || del|start|restart|stop <name>"
            " || raw"
-           " || sslcertkey"),
-        N_("         list: list relay clients (only active relays)\n"
-           "     listfull: list relay clients (verbose, all relays)\n"
-           "    listrelay: list relays (name and port)\n"
-           "          add: add a relay (listen on a port/path)\n"
-           "          del: remove a relay (clients remain connected)\n"
-           "        start: listen on port\n"
-           "      restart: close the server socket and listen again on port "
-           "(clients remain connected)\n"
-           "         stop: close the server socket (clients remain connected)\n"
-           "         name: relay name (see format below)\n"
-           "         port: port used for relay\n"
-           "         path: path used for relay (for UNIX domain socket only); "
-           "path is evaluated (see function string_eval_path_home in "
-           "plugin API reference)\n"
-           "          raw: open buffer with raw Relay data\n"
-           "   sslcertkey: set SSL certificate/key using path in option "
-           "relay.network.ssl_cert_key\n"
-           "\n"
-           "Relay name is: [ipv4.][ipv6.][ssl.]<protocol.name> or "
-           "unix.[ssl.]<protocol.name>\n"
-           "         ipv4: force use of IPv4\n"
-           "         ipv6: force use of IPv6\n"
-           "          ssl: enable SSL\n"
-           "         unix: use UNIX domain socket\n"
-           "protocol.name: protocol and name to relay:\n"
-           "                 - protocol \"irc\": name is the server to share "
-           "(optional, if not given, the server name must be sent by client in "
-           "command \"PASS\", with format: \"PASS server:password\")\n"
-           "                 - protocol \"weechat\" (name is not used)\n"
-           "\n"
-           "The \"irc\" protocol allows any IRC client (including WeeChat "
-           "itself) to connect on the port.\n"
-           "The \"weechat\" protocol allows a remote interface to connect on "
-           "the port, see the list here: https://weechat.org/about/interfaces/\n"
-           "\n"
-           "Without argument, this command opens buffer with list of relay "
-           "clients.\n"
-           "\n"
-           "Examples:\n"
-           "  irc proxy, for server \"libera\":\n"
-           "    /relay add irc.libera 8000\n"
-           "  irc proxy, for server \"libera\", with SSL:\n"
-           "    /relay add ssl.irc.libera 8001\n"
-           "  irc proxy, for all servers (client will choose), with SSL:\n"
-           "    /relay add ssl.irc 8002\n"
-           "  weechat protocol:\n"
-           "    /relay add weechat 9000\n"
-           "  weechat protocol with SSL:\n"
-           "    /relay add ssl.weechat 9001\n"
-           "  weechat protocol with SSL, using only IPv4:\n"
-           "    /relay add ipv4.ssl.weechat 9001\n"
-           "  weechat protocol with SSL, using only IPv6:\n"
-           "    /relay add ipv6.ssl.weechat 9001\n"
-           "  weechat protocol with SSL, using IPv4 + IPv6:\n"
-           "    /relay add ipv4.ipv6.ssl.weechat 9001\n"
-           "  weechat protocol over UNIX domain socket:\n"
-           "    /relay add unix.weechat ${weechat_runtime_dir}/relay_socket"),
+           " || tlscertkey"),
+        WEECHAT_CMD_ARGS_DESC(
+            N_("raw[list]: list relay clients (only active relays)"),
+            N_("raw[listfull]: list relay clients (verbose, all relays)"),
+            N_("raw[listrelay]: list relays (name and port)"),
+            N_("raw[add]: add a relay (listen on a port/path)"),
+            N_("raw[del]: remove a relay (clients remain connected)"),
+            N_("raw[start]: listen on port"),
+            N_("raw[restart]: close the server socket and listen again on port "
+               "(clients remain connected)"),
+            N_("raw[stop]: close the server socket (clients remain connected)"),
+            N_("name: relay name (see format below)"),
+            N_("port: port used for relay"),
+            N_("path: path used for relay (for UNIX domain socket only); "
+               "path is evaluated (see function string_eval_path_home in "
+               "plugin API reference)"),
+            N_("raw[raw]: open buffer with raw Relay data"),
+            N_("raw[tlscertkey]: set TLS certificate/key using path in option "
+               "relay.network.tls_cert_key"),
+            "",
+            N_("Relay name is: [ipv4.][ipv6.][tls.]<protocol.name> or "
+               "unix.[tls.]<protocol.name>:"),
+            N_("  - ipv4: force use of IPv4"),
+            N_("  - ipv6: force use of IPv6"),
+            N_("  - tls: enable TLS"),
+            N_("  - unix: use UNIX domain socket"),
+            N_("  - protocol.name: protocol and name to relay:"),
+            N_("    - protocol \"irc\": name is the server to share "
+               "(optional, if not given, the server name must be sent by client in "
+               "command \"PASS\", with format: \"PASS server:password\")"),
+            N_("    - protocol \"weechat\" (name is not used)"),
+            "",
+            N_("The \"irc\" protocol allows any IRC client (including WeeChat "
+               "itself) to connect on the port."),
+            N_("The \"weechat\" protocol allows a remote interface to connect on "
+               "the port, see the list here: https://weechat.org/about/interfaces/"),
+            "",
+            N_("Without argument, this command opens buffer with list of relay "
+               "clients."),
+            "",
+            N_("Examples:"),
+            AI("  /relay add irc.libera 8000"),
+            AI("  /relay add tls.irc.libera 8001"),
+            AI("  /relay add tls.irc 8002"),
+            AI("  /relay add weechat 9000"),
+            AI("  /relay add tls.weechat 9001"),
+            AI("  /relay add ipv4.tls.weechat 9001"),
+            AI("  /relay add ipv6.tls.weechat 9001"),
+            AI("  /relay add ipv4.ipv6.tls.weechat 9001"),
+            AI("  /relay add unix.weechat ${weechat_runtime_dir}/relay_socket")),
         "list %(relay_relays)"
         " || listfull %(relay_relays)"
         " || listrelay"
@@ -467,6 +460,6 @@ relay_command_init ()
         " || restart %(relay_relays)"
         " || stop %(relay_relays)"
         " || raw"
-        " || sslcertkey",
+        " || tlscertkey",
         &relay_command_relay, NULL, NULL);
 }
